// Standard Library
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <vector>

// External Libraries
#include <tbb/task_scheduler_init.h>
#include <tbb/tick_count.h>

// My Libraries
#include "assert.hpp"
#include "random_wrapper2.hpp"

// Internal 
#include "para.hpp"
#include "system.hpp"
#include "observer.hpp"

int main(int argc, char* argv[])
{
  Para::parse(argc, argv);

  /* I quadrupled n_gen because TBB sometimes uses more than the number
     of CPUs requested to tbb::task_scheduler_init */
  RandomWrapper2::set_seed(Para::seed, 4*Para::n_cpu);

  tbb::task_scheduler_init init(Para::n_cpu);

  System system;

  Observer observer(system);

  // /*debug*/
  // Histogram hist;
  // RandomTBB2::reference random = random_tbb_2.local();
  // double p[6] = {0.1, 0.1, 0.3, 0.1, 0.1, 0.1};
  // unsigned int n[6] = {0, 0, 0, 0, 0, 0};
  // random.Multinomial(6, (unsigned)1000000000, p, n);
  // for(long i=0; i<6; ++i){
  //   std::cout << i << ' ' << n[i] << std::endl;
  // }
  // exit(0);
  
  /* Count time */
  tbb::tick_count t0 = tbb::tick_count::now();
  
  for(unsigned long step=0; step <= Para::max_generation; ++step){

      observer.observe_system(step);
      if(observer.should_stop()){
	break;
      }
    
      system.run_one_step();
  }

  /* Count time */
  tbb::tick_count t1 = tbb::tick_count::now();
  double t = (t1-t0).seconds();
  std::cerr << "core simulation time(sec) = " << t << "\n";

  return 0;
}
