// Standard Library
#include <iostream>
#include <algorithm>
#include <vector>
#include <limits>


// External Libraries

// My Libraries
#include "assert.hpp"
#include "random_wrapper2.hpp"
#include "parallel-print.hpp"

// Internal 
#include "para.hpp"

#ifndef MOLECULE
#define MOLECULE

class Molecule{
  friend class Observer;
  

public:
  enum class Type {P, Q};

  Molecule();

  void set_k(double k);

  double get_k() const;

  void set_cell_index(std::size_t index);
  std::size_t get_cell_index() const;

  double get_fitness() const;
  void set_fitness(double _fitness);

  void replicate(const Molecule& rhs, RandomTBB2::reference random);

  void set_parent_index(std::size_t index);
  std::size_t get_parent_index() const;

private:
  void mutate();
  void mutate(RandomTBB2::reference random);
  void mutate_Full(RandomTBB2::reference random);

  void change_parameter_step_wrap(double& parameter,const double min,const double max,const double step, RandomTBB2::reference random);
  void change_parameter_negative(double& parameter,const double max,const double step, RandomTBB2::reference random);
  void change_parameter_no_boundary(double& parameter,const double step, RandomTBB2::reference random);
  void change_parameter_step_wrap_normal(double& parameter,const double min,const double max,const double step, RandomTBB2::reference random);
  void change_parameter_binary(double& parameter,const double min,const double max);
  void change_parameter_multi_locus(const double min, const double max, RandomTBB2::reference random);

  double genome2k() const;
  
private:
  double k;

  /* fitness to produce P */
  double fitness;

  /* Index of the Cell to which this Molecule belongs to. I use an index
     rather than a pointer because a pointer can be invalidated. */
  std::size_t cell_index;

  /* Index of the parent Molecule from which this Molecule has
     descended. This is not strictly necessary for the algorithm, but I
     use it for the purpose of debugging and observation. */
  std::size_t parent_index;

  std::vector<bool> genome;
};

std::ostream& operator<<(std::ostream& os, const Molecule::Type type);
std::ostream& operator<<(std::ostream& os, const Molecule& mol);
#endif
