#include "para.hpp"

namespace Para {
  unsigned long seed = 1236;
  unsigned n_cpu = 1;
  unsigned long division_threshold = 100;
  double kill_threshold = 0.25;
  unsigned long n_molecule = 5000;
  double mutation_rate = 0.01;
  double mutation_step = 0.1;
  unsigned long max_generation = std::numeric_limits<unsigned long>::max();
  std::string fname_plot = "plot.dat";
  std::string fname_cell_stat = "vesicle-stat.dat";
  std::string fname_etc = "";
  unsigned long plot_interval = 1000;
  unsigned long cell_stat_interval_factor = 10;
  unsigned long cell_stat_interval;
  double cell_stat_bin_width = 3e-3*Para::mutation_step;
  double init_k = 0.0;
  double selection_molecular = -1.0;
  double selection_cellular = 1.0;
  double max_k = 1.0;
  double min_k = 0.0;
  std::size_t n_loci = 100;
}

void Para::parse(int argc, char* argv[])
{
  try{
    namespace po = boost::program_options;
    po::options_description all_options("All options");
    all_options.add_options()
      ("help,h", 
       "Produce help message")

      ("seed",
       po::value<unsigned long>(&Para::seed)->default_value(Para::seed),
       "(part of) Random seed")

      ("n_cpu",
       po::value<unsigned>(&Para::n_cpu)->default_value(Para::n_cpu),
       "Number of CPUs (threads) used")

      ("max_generation",
       po::value<std::size_t>(&Para::max_generation)->default_value(Para::max_generation),
       "Maximum number of generations")

      ("n_molecule",
       po::value<unsigned long>(&Para::n_molecule)->default_value(Para::n_molecule),
       "Number of Molecules.")

      ("division_threshold",
       po::value<unsigned long>(&Para::division_threshold)->default_value(Para::division_threshold),
       "Division threshold.")

      ("mutation_rate",
       po::value<double>(&Para::mutation_rate)->default_value(Para::mutation_rate),
       "Mutation rate of recognition parameters.")

      ("mutation_step",
       po::value<double>(&Para::mutation_step)->default_value(Para::mutation_step),
       "mutation_step")

      ("plot_interval",
       po::value<unsigned long>(&Para::plot_interval)->default_value(Para::plot_interval),
       "Inverval for plot.dat")
 
      ("cell_stat_interval_factor",
       po::value<unsigned long>(&Para::cell_stat_interval_factor)->default_value(Para::cell_stat_interval_factor),
       "Inverval for vesicle-stat.dat will be set to factor * plot_interval. ")
 
      ("fname_plot",
       po::value<std::string>(&Para::fname_plot)->default_value(Para::fname_plot),
       "File name for plot data.")

      ("fname_cell_stat",
       po::value<std::string>(&Para::fname_cell_stat)->default_value(Para::fname_cell_stat),
       "File name for cell_stat data.")

      ("cell_stat_bin_width",
       po::value<double>(&Para::cell_stat_bin_width)->default_value(Para::cell_stat_bin_width),
       "Bin width of histogram in Cell statistics.")

      ("fname_etc",
       po::value<std::string>(&Para::fname_etc)->default_value(Para::fname_etc),
       "File name for debug data.")

      ("init_k",
       po::value<double>(&Para::init_k)->default_value(Para::init_k),
       "init_k")

      ("min_k",
       po::value<double>(&Para::min_k)->default_value(Para::min_k),
       "min_k")

      ("max_k",
       po::value<double>(&Para::max_k)->default_value(Para::max_k),
       "max_k")

      ("selection_molecular",
       po::value<double>(&Para::selection_molecular)->default_value(Para::selection_molecular),
       "Molecular-level selection coefficient")

      ("selection_cellular",
       po::value<double>(&Para::selection_cellular)->default_value(Para::selection_cellular),
       "Cellular-level selection coefficient")

      ("kill_threshold",
       po::value<double>(&Para::kill_threshold)->default_value(Para::kill_threshold),
       "kill_threshold")

      ("n_loci",
       po::value<unsigned long>(&Para::n_loci)->default_value(Para::n_loci),
       "Number of loci in a genome.")
      ;

    po::variables_map var_map;
    po::store(po::parse_command_line(argc, argv, all_options), var_map);
    po::notify(var_map);

    if(var_map.count("help")){
      std::cout << all_options;
      std::exit(0);
    }
  }
  catch(std::exception& e){
    std::cerr << "Para::parse() " << e.what() << "\n";
    std::exit(-1);
  }

  /* Set some parameters */
  Para::cell_stat_interval = Para::cell_stat_interval_factor * Para::plot_interval;

  if(Para::single_cell_mode){
    Para::division_threshold = Para::n_molecule + 1;
  }
}
