#include <cstdlib>
#include <cstddef>
#include <string>
#include <limits>
#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/program_options.hpp>

/*
  007: I added an option of multi-locus model.

  006: I added a feature to output information needed for ancestor
       tracking. Statistics for Cells, which are output if
       Para::trace_dynamics_of_each_cell==true, now include the indices
       of Cells and their parent Cells. To use this feature, set
       --plot_interval 1 --cell_stat_interval_factor 1
       --cell_stat_bin_width 100.

  005: Parallelized system::divide_cell(). The code is taken from
       plasmid. Also, plot.dat now includes covariance between w and k,
       and between w and (k-<k>)^2. Also, MutType::PerGenomebinary is
       added.

  004: I implemented Observer::print_cell_stat() and histogram.hpp.

  003: I implemented the 8-dimensional model. I also introduced
       exp_cash(), which speeds up the program by 50%.

  002: This program implements the hiereachical Wright-Fisher
       model. This is a newly created program entirely separately from
       001. I tried to parallelize the program, but was not very
       successful. When run in the parallel mode with n_cpu=2, the
       program can be slighly faster than when run in the serial if the
       total number of Molecules is very large. However, if n_cpu > 2,
       the program can actually be slower than in the serial mode. So,
       it's recommended to run with n_cpu=1.

  001: This seems unfinished and abandoned.
*/


#ifndef PARAMETERS
#define PARAMETERS
namespace Para {

  /* If set to true, it will print out statistics of Cells. See
     observer.cpp for details */
  const bool trace_dynamics_of_each_cell = true;

  /* Set to zero the fitness of the Molecules in the Cells that have
     less than kill_threshold*division_threshold Molecules. */
  const bool kill_small_cell = false;

  /* PerGenomeStepNoMinBoundary: x and y are NOT bounded below. x and y
     are bounded above with reflecting boundaries.

     PerGenomeStepNoMinMaxBoundary: x and y are NOT bounded below AND
     above.

     PerGenomeMultiLocus: a genome consists of n_loci loci. Each locus
     has either a plus (+1) or minus (-1) allele. The k value is the sum
     of alleles across all loci in a genome.
  */
  enum class MutType {PerGenomeStepWrap, PerGenomeStepNegative, PerGenomeStepNoMinBoundary, PerGenomeStepNoMinMaxBoundary, PerGenomeStepWrapNormal, PerGenomeBinary, PerGenomeMultiLocus};
  const MutType mut_type = MutType::PerGenomeMultiLocus;

  /* Set this true to disable cellular-level dynamics. This will also
     set Para::division_threshold = Para::n_molecule + 1. */
  const bool single_cell_mode = false;
  
  /* 
     Method of calculating fitness

        Neutral:

           w = 1.0

        Exp_Yukawa:

           w = exp(<k>*(max_k-k))

        Lin_Yukawa:

           w = <k>*(max_k-k)

        Exp_MNeutral:

           w = exp(<k>)

        Exp_Simple:

                             exp(sm*k)
           w = exp(sc*<k>) -------------
                            <exp(sm*k)>
  */
  enum class FitnessType {Neutral, Exp_Yukawa, Lin_Yukawa, Exp_Simple_MolecularNeutral, Exp_Simple};
  const FitnessType fitness_type = FitnessType::Exp_Simple;



  void parse(int argc, char* argv[]);

  extern unsigned long seed;
  extern unsigned n_cpu;
  extern unsigned long division_threshold;
  extern double kill_threshold;
  extern unsigned long n_molecule;
  extern double mutation_rate;
  extern double mutation_step;
  extern unsigned long max_generation;
  extern std::string fname_plot;
  extern std::string fname_cell_stat;
  extern std::string fname_etc;
  extern unsigned long plot_interval;
  extern unsigned long cell_stat_interval;
  extern double cell_stat_bin_width;
  extern double init_k;

  extern double selection_molecular;
  extern double selection_cellular;

  extern double max_k;
  extern double min_k;

  extern std::size_t n_loci;
}
#endif
