// Standard Library
#include <iostream>
#include <vector>
#include <random>

// External Libraries
#include <tbb/tbb.h>
#include <tbb/tbb_thread.h>
#include <tbb/partitioner.h>

// My Libraries
#include "assert.hpp"
#include "random_wrapper2.hpp"
#include "parallel-print.hpp"

// Internal 
#include "para.hpp"
#include "cell-pop.hpp"
#include "molecule.hpp"

#ifndef SYSTEM
#define SYSTEM

class System {
  friend class Observer;

public:
  System();

  // void test();
  void run_one_step();

private:
  void init();

  void calc_fitness();

  /* fisher_wright() will populate molvec_next_ptr by copying
     molvec_curr_ptr with mutation. */
  void fisher_wright();
  void fisher_wright_serial();
  void fisher_wright_parallel();
  class DoFisherWright;
  class SetFitnessVector;
  
  /* Assign Molecules in molvec_next_ptr to cellpop_next. It also
     updates the cell_index of Molecules in molvec_next. This method
     assumes that fisher_writhg() has already been called. */
  void divide_cell();
  void divide_cell_serial();
  void divide_cell_parallel();
  class DivideCell;

  std::size_t map_parent_cell_2_daughter_cells();

  /* Swap molvec_curr_ptr and molvec_next_ptr */
  void swap_generation();

  unsigned long get_n_generation() const;

  /* This check the consistency of the model's state. */
  bool check_consistency() const;

private:
  const bool nomultithread;
  /* The current generation switches between  molecules_0 and molecules_1. */
  typedef std::vector<Molecule> MolVec;
  MolVec molvec_0;
  MolVec molvec_1;
  MolVec* molvec_curr_ptr;
  MolVec* molvec_next_ptr;
  CellPop cellpop_0;
  CellPop cellpop_1;
  CellPop* cellpop_curr_ptr;
  CellPop* cellpop_next_ptr;
  unsigned long time;
  double fitness_sum;
  std::vector<double> fitness_vec;
};

#endif
